local OUTPUT = "|cffff4488CritSpit|r: |cffeda55f%s|r crit on |cffff0000%s|r for |cff00dbba%d|r."
local OUTPUT_NOTARGET = "|cffff4488CritSpit|r: |cffeda55f%s|r crit for |cff00dbba%d|r."
local WIPE = "|cffff4488CritSpit|r: Wiped all saved data."
local CHAT_OUTPUT = "CritSpit Record: %s crit for %d."
local TTHINT = "Use the slash command, |cffeda55f/critspit|r, to output crit records in channels."
local TTHEADING_ABILITY = "Ability"
local TTHEADING_DAMAGE = "Damage"
local HELP = {
	"/critspit all: Prints all your highest crits to the local chat frame.",
	"/critspit <ability name>: Prints the highest crit with that ability to your local chat frame.",
	"/critspit p/g/r/s <ability name>: Prints the highest crit to party, guild, raid chat or say.",
	"/critspit wipe: Wipes all the saved data.",
}

local filter = 0x001
local select = select

local addon = CreateFrame("Frame")
addon:SetScript("OnEvent", function(self, event, ...)
	self[event](self, ...)
end)

-- isCrit, amount, abilityName
local function spellDamage(...) return (select(9, ...)), (select(4, ...)), (select(2, ...)) end
local function swingDamage(...) return (select(6, ...)), (...), "Melee" end
local function spellHeal(...) return (select(5, ...)), (select(4, ...)), (select(2, ...)) end

local events = {
	SPELL_DAMAGE = spellDamage,
	SPELL_HEAL = spellHeal,
	SWING_DAMAGE = swingDamage, 
}

function addon:COMBAT_LOG_EVENT_UNFILTERED(_, eventType, _, _, srcFlags, _, dstName, _, ...)
	if not events[eventType] then return end
	if bit.band(srcFlags, filter) == 0 then return end
	local isCrit, amount, abilityName = events[eventType](...)
	if not isCrit or not abilityName or type(amount) ~= "number" then return end
	if not CritSpitDB[abilityName] or CritSpitDB[abilityName] < amount then
		CritSpitDB[abilityName] = amount
		DEFAULT_CHAT_FRAME:AddMessage(OUTPUT:format(abilityName, dstName, amount))
	end
end

function addon:ADDON_LOADED(msg)
	if msg == "Blizzard_CombatLog" then
		filter = COMBATLOG_OBJECT_AFFILIATION_MINE
	elseif msg == "CritSpit" then
		if not CritSpitDB then CritSpitDB = {} end
	end
end

addon:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
addon:RegisterEvent("ADDON_LOADED")

local channels = {
	p = "PARTY",
	g = "GUILD",
	r = "RAID",
	s = "SAY",
}
_G.SlashCmdList["CRITSPIT_SHORTHAND"] = function(input)
	input = input:trim()
	if input:len() == 0 then
		for i, v in ipairs(HELP) do
			DEFAULT_CHAT_FRAME:AddMessage(v)
		end
		return
	end
	if input:lower() == "all" then
		for k, v in pairs(CritSpitDB) do
			DEFAULT_CHAT_FRAME:AddMessage(OUTPUT_NOTARGET:format(k, v))
		end
	elseif input:lower() == "wipe" then
		CritSpitDB = {}
		DEFAULT_CHAT_FRAME:AddMessage(WIPE)
	elseif input:find("^%l%s") then
		local channel, ability = select(3, input:find("^(%l)%s(.*)"))
		if not channels[channel] then return end
		if not CritSpitDB[ability] then return end
		SendChatMessage(CHAT_OUTPUT:format(ability, CritSpitDB[ability]), channels[channel])
	elseif CritSpitDB[input] then
		DEFAULT_CHAT_FRAME:AddMessage(CHAT_OUTPUT:format(input, CritSpitDB[input]))
	end
end
_G.SLASH_CRITSPIT_SHORTHAND1 = "/critspit"

if LibStub then
	local ldb = LibStub:GetLibrary("LibDataBroker-1.1", true)
	if ldb then
		local csLDB = ldb:NewDataObject("CritSpit", {
			type = "data source",
			text = "CritSpit",
			icon = "Interface\\Icons\\Spell_Shadow_AbominationExplosion",
		})
		function csLDB.OnTooltipShow(tt)
			tt:AddLine("CritSpit")
			if next(CritSpitDB) then
				tt:AddDoubleLine(TTHEADING_ABILITY, TTHEADING_DAMAGE, 0.7, 0.8, 0, 1, 0, 0)
				for k, v in pairs(CritSpitDB) do
					tt:AddDoubleLine(k, v, 1, 1, 1, 1, 0, 0)
				end
			end
			tt:AddLine(TTHINT, 0.2, 1, 0.2, 1)
		end
	end
end

